#include <iostream>
#include <fstream>
#include <string>

using namespace std;

template<typename T>
class BinTree {
private:
	struct Node {
		T data;
		unsigned int cnt;
		Node *left, *right;
	} *root;
	Node *insert(Node *node, T key); 
	bool isElem(Node *node, T key);
	void clear(Node *node);
public:
	BinTree()        { root = nullptr; }
	BinTree(string& filename);
	void insert(T x) { root = insert(root, x); }
	bool isElem(T x) { return isElem(root, x); }
	void clear()     { clear(root); root = nullptr; }
	void print()     { print(root); }
	void print(Node *node);
	~BinTree()       { clear(); }
};

template<typename T>
BinTree<T>::BinTree(string& filename) : BinTree() {
	ifstream source;
	source.open(filename.c_str());
	if (!source.is_open()) exit(1);
	T s;
	while (!source.eof()) {
		source >> s;
		insert(s);
	}
	source.close();
}

template<typename T>
bool BinTree<T>::isElem(Node *node, T key) {
	if (node == nullptr) return false;
	if (node->data == key) return true;
	if (node->data < key) return isElem(node->right, key);
	return isElem(node->left, key);
}
template<typename T>
void BinTree<T>::clear(Node *node) {
	if (node == nullptr) return;  // Rekursionsabbruch
	clear(node->left);            // linken Unterbaum löschen
	clear(node->right);           // rechten Unterbaum löschen
	delete node;                  // Knoten löschen
}
template<typename T>
typename BinTree<T>::Node* BinTree<T>::insert(Node *node, T key) {
	if (node == nullptr) {
		node = new Node;
		node->data = key;
		node->cnt = 1;
		node->left = node->right = nullptr;
		return node;
	}
	if (node->data < key) 
		node->right = insert(node->right, key);
	else if (node->data > key)
		node->left = insert(node->left, key);
	else
		node->cnt++;
	return node;
}
template<typename T>
void BinTree<T>::print(Node *node) {
	if (node == nullptr) return;
	print(node->left);
	cout << node->cnt << " " << node->data.c_str() << endl;
	print(node->right);
}
